//  ===================================================================
//
//  Program:   Quadrilateral Mesh Post-Processor
//  Module:    reader.h
//  Language:  C++
//  Date:      March 2003
//  Version:   1.0.0
//
//  Copyright (c) 2002, 2003
//
//  Marcelo Siqueira
//
//  Graduate Student
//  Department of Computer and Information Science
//  Moore School Building
//  200 South 33rd Street, Suite #62
//  Philadelphia, PA 19104-6389
//  Phone No.: (215) 898-5879
//
//  marcelos@seas.upenn.edu
//
// This program  is distributed  WITHOUT ANY WARRANTY,  and it  may be
// freely redistributed under the condition that the copyright notices
// are  not  removed,  and  no  compensation  is  received.   Private,
// research, and institutional use  is free. Distribution of this code
// as  part of  a  commercial  system is  permissible  ONLY BY  DIRECT
// ARRANGEMENT WITH THE AUTHOR.
//
//  ==================================================================

#ifndef __edgekey_h
#define __edgekey_h

namespace qmpp {

  // -----------------------------------------------------------------
  // class tEdgeKey
  //
  // This class implements a way of coding edges of a 2D mesh.
  //
  // -----------------------------------------------------------------
  class tEdgeKey {
  public:
    // Constructors
    tEdgeKey() :
      m_E1(-1),
      m_E2(-1)
    {}

    tEdgeKey(int e1, int e2) :
      m_E1(e1),
      m_E2(e2)
    {}

    tEdgeKey(const tEdgeKey& k) :
      m_E1(k.m_E1),
      m_E2(k.m_E2)
    {}
  
    // less than operator 
    bool operator<(const tEdgeKey& k) const {
      if (m_E1 == k.m_E1) return m_E2 < k.m_E2;
      return m_E1 < k.m_E1;
    }
  	
    // Get methods
    int getEdgeC1() const
      { return m_E1; }
  		
    int getEdgeC2() const
      { return m_E2; }

    // Computes reverse edge key
    tEdgeKey reverse() const
      { return tEdgeKey(m_E2,m_E1); }

   private:
     int m_E1;
     int m_E2;
  };

};     // end namespace qmpp

#endif   // __edgekey_h
