//  ===================================================================
//
//  Program:   Quadrilateral Mesh Post-Processor
//  Module:    polygon2.h
//  Language:  C++
//  Date:      March 2003
//  Version:   1.0.0
//
//  Copyright (c) 2002, 2003
//
//  Marcelo Siqueira
//
//  Graduate Student
//  Department of Computer and Information Science
//  Moore School Building
//  200 South 33rd Street, Suite #62
//  Philadelphia, PA 19104-6389
//  Phone No.: (215) 898-5879
//
//  marcelos@seas.upenn.edu
//
// This program  is distributed  WITHOUT ANY WARRANTY,  and it  may be
// freely redistributed under the condition that the copyright notices
// are  not  removed,  and  no  compensation  is  received.   Private,
// research, and institutional use  is free. Distribution of this code
// as  part of  a  commercial  system is  permissible  ONLY BY  DIRECT
// ARRANGEMENT WITH THE AUTHOR.
//
//  ==================================================================

#ifndef __polygon2_h
#define __polygon2_h

#include <vector>

#include <iostream>
#include <sstream>

#include "typedefs.h"
#include "exceptionobj.h"

using namespace std;

namespace qmpp {

  // -----------------------------------------------------------------
  // class tIndVertex2
  //
  // This  class   implements  vertices  in  2D.   Such  vertices  are
  // represented by a location and an ID number.
  //
  // -----------------------------------------------------------------

  class tIndVertex2 {
  public:
    // Constructors
    tIndVertex2() : 
      m_id(-1),
      m_v(Point2(0,0))
    {}

    tIndVertex2(int id, const Point2& p) : 
      m_id(id),
      m_v(p)
    {}

    tIndVertex2(const tIndVertex2& v)
      {
	m_id = v.m_id;
	m_v  = v.m_v;
      }

    // Get methods
    int getVertexId() const
      { return m_id; }

    Point2 getPoint() const
      { return m_v; }

    // Set methods
    void setVertexId(int id)
      { m_id = id; }

    void setPoint(const Point2& p)
      { m_v = p; }

  protected:
    // ID of this vertex
    int m_id;
 
    // Location of this vertex
    Point2 m_v;
  };


  // -----------------------------------------------------------------
  // class tPolygon
  //
  // This class represents simple polygons in 2D.
  //
  // -----------------------------------------------------------------
  class tPolygon2 {
  public:
    // Constructors
    tPolygon2();
    tPolygon2(int);
    tPolygon2(const tPolygon2&);

    virtual ~tPolygon2() {};

    // Assignment operator
    tPolygon2& operator=(const tPolygon2&);

    // Get methods
    int getNumVerts() const
      { return m_size; }

    int getVertexId(int i) const
      { return m_verts[i].getVertexId(); }

    Point2 getPoint(int i) const
      { return m_verts[i].getPoint(); }

    tIndVertex2 getIndVertex(int i) const
      { return m_verts[i]; }

    int findReflexPoint() const throw (tExceptionObject);
    int getIndexOfPoint(const Point2&) const throw (tExceptionObject);
    int getNumOfReflexPoints() const;
    virtual double area() const;

    // Set methods
    void setConstraint(int i,bool e)
      { m_ec[i] = e; }

    // computes cosine of the angle  defined by a given vertex of this
    // polygon.
    double cosangle(int) const throw (tExceptionObject);

    // Query methods
    virtual bool isCW() const;
    virtual bool isCCW() const;
    virtual bool isConvex() const;
    virtual bool isReflex(int) const;

    virtual bool isConstrained(int i) const
      { return m_ec[i]; }

    virtual bool isSimple() const;

    // Modifiers
    virtual void reverse();
    virtual void insert(int, int, const Point2&);
    virtual void insert(int, const tIndVertex2&);
    virtual void shiftLeft(int);
    virtual void shiftRight(int);

  protected:
    // Vertices of this polygon
    vector<tIndVertex2> m_verts;

    // "m_ec[i]"  is  true if  the  edge (m_verts(i),m_verts(i+1))  is
    // constrained. Otherwise, it is false.
    vector<bool> m_ec;

    // Current number of vertices of this polygon
    int m_size;
  };


};

#endif   // __polygon2_h
