//  ===================================================================
//
//  Program:   Quadrilateral Mesh Post-Processor
//  Module:    smartptr.cc
//  Language:  C++
//  Date:      March 2003
//  Version:   1.0.0
//
//  Copyright (c) 2002, 2003
//
//  Marcelo Siqueira
//
//  Graduate Student
//  Department of Computer and Information Science
//  Moore School Building
//  200 South 33rd Street, Suite #62
//  Philadelphia, PA 19104-6389
//  Phone No.: (215) 898-5879
//
//  marcelos@seas.upenn.edu
//
// This program  is distributed  WITHOUT ANY WARRANTY,  and it  may be
// freely redistributed under the condition that the copyright notices
// are  not  removed,  and  no  compensation  is  received.   Private,
// research, and institutional use  is free. Distribution of this code
// as  part of  a  commercial  system is  permissible  ONLY BY  DIRECT
// ARRANGEMENT WITH THE AUTHOR.
//
//  ==================================================================

#include "smartptr.h"

// -------------------------------------------------------------------
// Method tSmartPtr()
// 
// Default constructor
//
// -------------------------------------------------------------------
template <class T>
tSmartPtr<T>::tSmartPtr(T* ptr)
{
  m_pointee = ptr;
  if (m_pointee) m_pointee->addRef(); 
}

// -------------------------------------------------------------------
// Method tSmartPtr()
// 
// Copy constructor
//
// -------------------------------------------------------------------
template <class T>
tSmartPtr<T>::tSmartPtr(const tSmartPtr<T>& sptr)
{
  m_pointee = sptr.m_pointee;
  if (m_pointee) m_pointee->addRef();
}

// -------------------------------------------------------------------
// Method ~tSmartPtr()
// 
// Destructor
//
// -------------------------------------------------------------------
template <class T>
tSmartPtr<T>::~tSmartPtr<T>()
{
  release();
}

// -------------------------------------------------------------------
// Method operator=()
// 
// Assignment operator
//
// -------------------------------------------------------------------
template <class T>
tSmartPtr<T>&
tSmartPtr<T>::operator=(const tSmartPtr<T>& sptr)
{
  T* temp = m_pointee;
  if (temp) temp->release();

  m_pointee = sptr.m_pointee;

  if (m_pointee) m_pointee->addRef();

  return *this;
}

// -------------------------------------------------------------------
// Method operator*()
// 
// Deferencing operator
//
// -------------------------------------------------------------------
template <class T>
T&
tSmartPtr<T>::operator*() const
{
  return *(m_pointee);
}

// -------------------------------------------------------------------
// Method operator->()
// 
// Deferencing operator
//
// -------------------------------------------------------------------
template <class T>
T*
tSmartPtr<T>::operator->() const
{
  return m_pointee;
}

// -------------------------------------------------------------------
// Method operator==()
// 
// Equality operator
//
// -------------------------------------------------------------------
template <class T>
bool
tSmartPtr<T>::operator==(const tSmartPtr<T>& sptr) const
{
  return m_pointee == sptr.m_pointee;
}

// -------------------------------------------------------------------
// Method operator!=()
// 
// Inequality operator
//
// -------------------------------------------------------------------
template <class T>
bool
tSmartPtr<T>::operator!=(const tSmartPtr<T>& sptr) const
{
  return m_pointee != sptr.m_pointee;
}

// -------------------------------------------------------------------
// Method operator!=()
// 
// Test nulliness of the pointee
//
// -------------------------------------------------------------------
template <class T>
bool
tSmartPtr<T>::operator!() const
{
  return (m_pointee == 0);
}

// -------------------------------------------------------------------
// Method isNotNull()
// 
// Test nulliness of the pointee
//
// -------------------------------------------------------------------
template <class T>
bool
tSmartPtr<T>::isNotNull() const
{
  return (m_pointee != 0);
}

// -------------------------------------------------------------------
// Method isNull()
// 
// Test nulliness of the pointee
//
// -------------------------------------------------------------------
template <class T>
bool
tSmartPtr<T>::isNull() const
{
  return (m_pointee == 0);
}

// -------------------------------------------------------------------
// Method getPointee()
// 
// Returns the pointee
//
// -------------------------------------------------------------------
template <class T>
T*
tSmartPtr<T>::getPointee() const
{
  return m_pointee;
}

// -------------------------------------------------------------------
// Method release()
// 
// Decrements reference count of the pointee, and calls its destructor
// if needed.
//
// -------------------------------------------------------------------
template <class T>
void
tSmartPtr<T>::release()
{
  T* temp = m_pointee;

  if (temp) temp->release();

  m_pointee = 0;
}

#ifdef QMPP_MANUAL_INSTANTIATION
#include "smartptr.txx"
#endif
