//  ===================================================================
//
//  Program:   Quadrilateral Mesh Post-Processor
//  Module:    smartptr.h
//  Language:  C++
//  Date:      March 2003
//  Version:   1.0.0
//
//  Copyright (c) 2002, 2003
//
//  Marcelo Siqueira
//
//  Graduate Student
//  Department of Computer and Information Science
//  Moore School Building
//  200 South 33rd Street, Suite #62
//  Philadelphia, PA 19104-6389
//  Phone No.: (215) 898-5879
//
//  marcelos@seas.upenn.edu
//
// This program  is distributed  WITHOUT ANY WARRANTY,  and it  may be
// freely redistributed under the condition that the copyright notices
// are  not  removed,  and  no  compensation  is  received.   Private,
// research, and institutional use  is free. Distribution of this code
// as  part of  a  commercial  system is  permissible  ONLY BY  DIRECT
// ARRANGEMENT WITH THE AUTHOR.
//
//  ==================================================================

#ifndef __smartptr_h_
#define __smartptr_h_

// -------------------------------------------------------------------
// class tSmartPtr 
//
// This  class is  an implementation  of smart  pointers  according to
// Scott  Meyers'  book "More  Effective  C++", Addison-Wesley,  1996,
// pp. 159-182.
//
// -------------------------------------------------------------------

template <class T>
class tSmartPtr {
public:
  // Constructors
  explicit tSmartPtr<T>(T* ptr = 0);
  tSmartPtr<T>(const tSmartPtr<T>&);

  // Destructor
  ~tSmartPtr<T>();

  // Assignment operator
  tSmartPtr<T>& operator=(const tSmartPtr<T>&); 

  // Deferencing operators
  T& operator*() const;
  T* operator->() const; 

  // Overload operator to test equality
  bool operator==(const tSmartPtr<T>&) const;

  // Overload operator to test equality
  bool operator!=(const tSmartPtr<T>&) const;

  // Test nulliness of the pointee
  bool operator!() const;
  bool isNotNull() const;
  bool isNull() const;

  // Get method
  T* getPointee() const;

private:
  void release(); 

  //
  // Private Attributes
  //

  // Actual pointer to an object of type T
  T* m_pointee;

};

#endif    // __smartptr_h_
