//  ===================================================================
//
//  Program:   Quadrilateral Mesh Post-Processor
//  Module:    boundary2.h
//  Language:  C++
//  Date:      March 2003
//  Version:   1.0.0
//
//  Copyright (c) 2002, 2003
//
//  Marcelo Siqueira
//
//  Graduate Student
//  Department of Computer and Information Science
//  Moore School Building
//  200 South 33rd Street, Suite #62
//  Philadelphia, PA 19104-6389
//  Phone No.: (215) 898-5879
//
//  marcelos@seas.upenn.edu
//
// This program  is distributed  WITHOUT ANY WARRANTY,  and it  may be
// freely redistributed under the condition that the copyright notices
// are  not  removed,  and  no  compensation  is  received.   Private,
// research, and institutional use  is free. Distribution of this code
// as  part of  a  commercial  system is  permissible  ONLY BY  DIRECT
// ARRANGEMENT WITH THE AUTHOR.
//
//  ==================================================================

#ifndef __boundary2_h
#define __boundary2_h

#include "typedefs.h"
#include "macros.h"
#include "smartptr.h"
#include "dataobj.h"

#include "edge2.h"

namespace hedge {

  // -----------------------------------------------------------------
  // Forward definition of class
  // -----------------------------------------------------------------
  class tComplex2;

  // -----------------------------------------------------------------
  // class tBoundary2 
  //
  // This  class implements  boundary curves  in the  handle-edge data
  // structure.
  //
  // -----------------------------------------------------------------

  class tBoundary2 : public tDataObject {
  public:
    // Standard class typedefs
    typedef tBoundary2            Self;
    typedef tSmartPtr<Self>       Pointer;
    typedef tSmartPtr<const Self> ConstPointer;

    // Type definition
    typedef tEdge2::Pointer egPointer;

    // Method for creating an instance of this object
    newMacro(Self);

    // Constructor
    tBoundary2() {}

    // Destructor
    ~tBoundary2() {}

    // Returns a smart pointer to a boundary edge in this boundary
    egPointer getBoundaryEdge() const
      { return m_edge; }

    // Sets a value to the pointer to a boundary edge of this boundary
    void setBoundaryEdge(const egPointer& e) 
      { m_edge = e; }

    // Friend class
    //
    // This is the only class  whose methods can use the private "set"
    // methods of the class tBoundary2.
    //
    friend class tComplex2;

  private:
    // Purposely not implemented
    tBoundary2(const Self&);
    void operator=(const Self&);

    //
    // Private attributes
    //

    // One edge in this boundary curve
    egPointer m_edge;
  };

};    // end namespace hedge


#endif   // __boundary2_h
