//  ===================================================================
//
//  Program:   Quadrilateral Mesh Post-Processor
//  Module:    dualgraph.cc
//  Language:  C++
//  Date:      March 2003
//  Version:   1.0.0
//
//  Copyright (c) 2002, 2003
//
//  Marcelo Siqueira
//
//  Graduate Student
//  Department of Computer and Information Science
//  Moore School Building
//  200 South 33rd Street, Suite #62
//  Philadelphia, PA 19104-6389
//  Phone No.: (215) 898-5879
//
//  marcelos@seas.upenn.edu
//
// This program  is distributed  WITHOUT ANY WARRANTY,  and it  may be
// freely redistributed under the condition that the copyright notices
// are  not  removed,  and  no  compensation  is  received.   Private,
// research, and institutional use  is free. Distribution of this code
// as  part of  a  commercial  system is  permissible  ONLY BY  DIRECT
// ARRANGEMENT WITH THE AUTHOR.
//
// ===================================================================

#include "dualgraph.h"

namespace hedge {

  // -----------------------------------------------------------------
  // Method: addNeighbor
  // 
  // Adds a adjacent vertex to the set of neighbors of this vertex.
  //
  // -----------------------------------------------------------------
  void
  tDualGraphVertex::addNeighbor(const tDualGraphVertex::Pointer& neigh)
  {
    if (m_degree == m_maxdg) m_neigh.resize(m_neigh.capacity() << 1);
 
    m_neigh[m_degree] = neigh;
    ++m_degree;
  }

  // -----------------------------------------------------------------
  // Method: delNeighbor
  // 
  // Removes  a adjacent  vertex from  the  set of  neighbors of  this
  // vertex.
  //
  // -----------------------------------------------------------------
  void
  tDualGraphVertex::delNeighbor(const tDualGraphVertex::Pointer& v)
  {
    int i = 0;
    while ((i < m_degree) && (m_neigh[i] != v)) {
      i++;
    }

    if (i < m_degree) {
      for (int j=i; j<m_degree-1; j++) {
	m_neigh[j] = m_neigh[j+1];
      }

      --m_degree; --v->m_degree;
    }
  }

  // -----------------------------------------------------------------
  // Method: delVert()
  // 
  // Removes a vertex from the set of vertices of this dual graph.
  //
  // -----------------------------------------------------------------
  void
  tDualGraph::delVert(const tDualGraphVertex::Pointer& v)
  {
    // for each neighbor, remove adjacency information
    int dg = v->getDegree();
    for (int i=0; i<dg; i++) {
      v->getNeighbor(i)->delNeighbor(v);
    }

    m_lverts.remove(v);
  }

  // -----------------------------------------------------------------
  // Method: clearMark()
  // 
  // Sets all vertices of this dual graph as unmarked ones.
  //
  // -----------------------------------------------------------------
  void 
  tDualGraph::clearMark() {
    list<dvPointer>::iterator vi;

    for (vi = m_lverts.begin(); vi != m_lverts.end(); ++vi) {
      (*vi)->setMark(false);
    }
  }

};   // end namespace hedge
