//  ===================================================================
//
//  Program:   Quadrilateral Mesh Post-Processor
//  Module:    halfedge2.h
//  Language:  C++
//  Date:      March 2003
//  Version:   1.0.0
//
//  Copyright (c) 2002, 2003
//
//  Marcelo Siqueira
//
//  Graduate Student
//  Department of Computer and Information Science
//  Moore School Building
//  200 South 33rd Street, Suite #62
//  Philadelphia, PA 19104-6389
//  Phone No.: (215) 898-5879
//
//  marcelos@seas.upenn.edu
//
// This program  is distributed  WITHOUT ANY WARRANTY,  and it  may be
// freely redistributed under the condition that the copyright notices
// are  not  removed,  and  no  compensation  is  received.   Private,
// research, and institutional use  is free. Distribution of this code
// as  part of  a  commercial  system is  permissible  ONLY BY  DIRECT
// ARRANGEMENT WITH THE AUTHOR.
//
//  ==================================================================

#ifndef __halfedge2_h
#define __halfedge2_h

#include "macros.h"
#include "smartptr.h"
#include "dataobj.h"

#include "vertex2.h"

namespace hedge {

  // -----------------------------------------------------------------
  // Forward definition of classes
  // -----------------------------------------------------------------

  class tComplex2 ;
  class tEdge2 ;
  class tFace2 ;

  // -----------------------------------------------------------------
  // class tHalfEdge2 
  //
  // This  class   implements  half-edges  in   the  handle-edge  data
  // structure.
  //
  // -----------------------------------------------------------------

  class tHalfEdge2 : public tDataObject {
  public:
    // Standard class typedefs
    typedef tHalfEdge2            Self;
    typedef tSmartPtr<Self>       Pointer;
    typedef tSmartPtr<const Self> ConstPointer;

    // Type definitions
    typedef tVertex2::Pointer     vtPointer;
    typedef tSmartPtr< tEdge2 >   egPointer;
    typedef tSmartPtr< tFace2 >   fcPointer;

    // Method for creating an instance of this object
    newMacro( Self ) ;

    // Virtual destructor
    ~tHalfEdge2() {}

    // Returns a smart pointer to the initial vertex of this half-edge
    vtPointer getVertex() const
      { return m_vert; }

    // Returns a smart pointer to the edge containing this half-edge
    egPointer getEdge() const
      { return m_edge; }

    // Returns a smart pointer to the face containing this half-edge
    fcPointer getFace() const
      { return m_face; }

    // Returns a smart pointer to  the half-edge following this one in
    // their face cycle.
    Pointer getNext() const
      { return m_next; }

    // Returns a smart pointer to  the half-edge preceding this one in
    // their face cycle.
    Pointer getPrev() const
      { return m_prev; }

    // Sets the initial vertex of this half-edge
    void setVertex(const vtPointer& v)
      { m_vert = v; }

    // Sets the edge containing this half-edge
    void setEdge(const egPointer& e)
      { m_edge = e; }

    // Sets the face containing this half-edge
    void setFace(const fcPointer& f)
      { m_face = f; }

    // Sets the successor half-edge of this one in their face cycle
    void setNext(const Pointer& he)
      { m_next = he; }

    // Sets the predecessor half-edge of this one in their face cycle
    void setPrev(const Pointer& he)
      { m_prev = he; }

    // Friend class
    //
    // This is the only class  whose methods can use the private "set"
    // methods of the class tHalfEdge2.
    //
    friend class tCQMComplex2;

  private:
    // purposely not implemented
    tHalfEdge2() {}
    tHalfEdge2(const Self&);
    void operator=(const Self&);

    //
    // Private attributes
    //

    // Initial vertex of this halfedge
    vtPointer m_vert;

    // Edge containing this half-edge
    egPointer m_edge;

    // Face containing this half-edge
    fcPointer m_face;

    // Next half-edge in the face cycle
    Pointer m_next;

    // Previous half-edge in the face cycle
    Pointer m_prev;
  };

};   // end namespace hedge


#endif   // __halfedge2_h
