//  ===================================================================
//
//  Program:   Quadrilateral Mesh Post-Processor
//  Module:    vertex2.h
//  Language:  C++
//  Date:      March 2003
//  Version:   1.0.0
//
//  Copyright (c) 2002, 2003
//
//  Marcelo Siqueira
//
//  Graduate Student
//  Department of Computer and Information Science
//  Moore School Building
//  200 South 33rd Street, Suite #62
//  Philadelphia, PA 19104-6389
//  Phone No.: (215) 898-5879
//
//  marcelos@seas.upenn.edu
//
// This program  is distributed  WITHOUT ANY WARRANTY,  and it  may be
// freely redistributed under the condition that the copyright notices
// are  not  removed,  and  no  compensation  is  received.   Private,
// research, and institutional use  is free. Distribution of this code
// as  part of  a  commercial  system is  permissible  ONLY BY  DIRECT
// ARRANGEMENT WITH THE AUTHOR.
//
//  ==================================================================

#ifndef __vertex2_h
#define __vertex2_h

#include "typedefs.h"
#include "macros.h"
#include "smartptr.h"
#include "dataobj.h"

#include "vertex2attrib.h"

namespace hedge {

  // -----------------------------------------------------------------
  // Forward definition of classes
  // -----------------------------------------------------------------
  class tComplex2;
  class tHalfEdge2;

  // -----------------------------------------------------------------
  // class tVertex2 
  //
  // This  class  implements  2D  vertices  in  the  handle-edge  data
  // structure.
  //
  // -----------------------------------------------------------------

  class tVertex2 : public tDataObject {
  public:
    // Standard class typedefs
    typedef tVertex2              Self;
    typedef tSmartPtr<Self>       Pointer;
    typedef tSmartPtr<const Self> ConstPointer;

    // Type definitions
    typedef tSmartPtr<tHalfEdge2> hePointer;
    typedef tVertex2Attrib::Pointer vaPointer;

    // Method for creating an instance of this object
    newMacro(Self);

    // Default constructor
    tVertex2() {}

    // Virtual destructor
    virtual ~tVertex2() {}

    // Returns the location of this vertex
    Point2 getPoint() const
      { return m_point; }

    // Returns a smart pointer to the initial half-edge of this vertex
    hePointer getHalfEdge() const
      { return m_shedge; }

    // Returns a smart pointer to  a boundary half-edge with origin at
    // this vertex, if any.
    hePointer getBoundHalfEdge() const
      { return m_bhedge; }

    // Returns a smart pointer to the attributes of this vertex
    vaPointer getAttribute() const
      { return m_att; }

    // Assigns attributes to this vertex
    void setAttribute(const vaPointer& att)
      {  m_att = att; }

    // Returns true  if this  object is a  boundary vertex,  and false
    // otherwise.
    bool isOnBoundary() const
      { return m_bhedge.isNotNull(); }

    // Sets the location of this vertex
    void setPoint(const Point2& p)
      { m_point = p; }

    // Sets the initial half-edge of this vertex
    void setHalfEdge(const hePointer& she)
      { m_shedge = she; }

    // Sets a boundary half-edge with origin at this vertex, if any
    void setBoundHalfEdge(const hePointer& bhe)
      { m_bhedge = bhe; }

    // Returns  true if  this  vertex  is a  vertex  in a  constrained
    // edge. Otherwise, it returns false.
    bool isOnConstrainedEdge() const;

    // Friend class
    //
    // This is the only class  whose methods can use the private "set"
    // methods of the class tVertex2.
    //
    friend class tComplex2;

  private:
    // Purposely not implemented
    tVertex2(const Self&);
    void operator=(const Self&);

    //
    // Private attributes
    //

    // Location of the vertex
    Point2 m_point;

    // Smart pointer to some halfedge whose initial vertex is this vertex.
    hePointer m_shedge;

    // Smart pointer  to the boundary  half edge whose  initial vertex
    // this vertex, if any.
    hePointer m_bhedge;

    // Smart pointer to the attributes of this vertex, if any
    vaPointer m_att;
  };


};   // end namespace hedge


#endif   // __vertex2_h
