//  ===================================================================
//
//  Program:   Quadrilateral Mesh Post-Processor
//  Module:    lexer.cc
//  Language:  C++
//  Date:      March 2003
//  Version:   1.0.0
//
//  Copyright (c) 2002, 2003
//
//  Marcelo Siqueira
//
//  Graduate Student
//  Department of Computer and Information Science
//  Moore School Building
//  200 South 33rd Street, Suite #62
//  Philadelphia, PA 19104-6389
//  Phone No.: (215) 898-5879
//
//  marcelos@seas.upenn.edu
//
// This program  is distributed  WITHOUT ANY WARRANTY,  and it  may be
// freely redistributed under the condition that the copyright notices
// are  not  removed,  and  no  compensation  is  received.   Private,
// research, and institutional use  is free. Distribution of this code
// as  part of  a  commercial  system is  permissible  ONLY BY  DIRECT
// ARRANGEMENT WITH THE AUTHOR.
//
//  ==================================================================

#include "lexer.h"

namespace qmpp {

  // -----------------------------------------------------------------
  // Method: throw_error() 
  // 
  // Throws an exception whose description contains the line number of
  // the input/output file that causes the error.
  //
  // -----------------------------------------------------------------
  void
  tLexer::throw_error(const string& s) throw (tExceptionObject) {
    stringstream ss (stringstream::in | stringstream::out);

    ss << "File line " << linecount_ << ": " << s;
    throw tExceptionObject(__FILE__,__LINE__,ss.str()); 
  }

  // -----------------------------------------------------------------
  // Method: increment_line_()
  // 
  // Increments the line counter of the lexical analyzer
  //
  // -----------------------------------------------------------------
  void
  tLexer::increment_line_() {
    if (linecount_ >= 0) {
      ++linecount_;
    }
  }

  // -----------------------------------------------------------------
  // Method: skip_space_()
  // 
  // Used by  the lexical analyzer  to ignore "ponctuation"  and space
  // tokens.
  //
  // -----------------------------------------------------------------
  void
  tLexer::skip_space_() {
    char c;
    for (;;) {
      if (!real_stream_.get(c)) {
	return;
      }

      if (c == '\n') {
	increment_line_();
      }

      if (!isspace(c)) {
	real_stream_.putback(c);
	return;
      }
    }
  }

  // -----------------------------------------------------------------
  // Method: get_int()
  // 
  // Reads an integer number from the file.
  //
  // -----------------------------------------------------------------
  bool
  tLexer::get_int(int& i) {
    skip_space_();
    char c;
    if (!real_stream_.get(c)) {
      throw_error("Unexpected eof in the middle of reading an int");
    }

    real_stream_.putback(c);
    if (!(real_stream_ >> i)) { 
      throw_error("Error while reading integer");
    }

    return true;
  }

  // -----------------------------------------------------------------
  // Method: get_double()
  // 
  // Reads a real number from the file.
  //
  // -----------------------------------------------------------------
  bool
  tLexer::get_double(double& d) {
    skip_space_();
    char c;
    if (!real_stream_.get(c)) {
      throw_error("Unexpected eof in the middle of reading a double");
    }
   
    real_stream_.putback(c);
    if (!(real_stream_ >> d)) {
      throw_error("Error while reading double");
    }

    return true;
  }

};    // end namespace qmpp
