//  ===================================================================
//
//  Program:   Quadrilateral Mesh Post-Processor
//  Module:    lexer.h
//  Language:  C++
//  Date:      March 2003
//  Version:   1.0.0
//
//  Copyright (c) 2002, 2003
//
//  Marcelo Siqueira
//
//  Graduate Student
//  Department of Computer and Information Science
//  Moore School Building
//  200 South 33rd Street, Suite #62
//  Philadelphia, PA 19104-6389
//  Phone No.: (215) 898-5879
//
//  marcelos@seas.upenn.edu
//
// This program  is distributed  WITHOUT ANY WARRANTY,  and it  may be
// freely redistributed under the condition that the copyright notices
// are  not  removed,  and  no  compensation  is  received.   Private,
// research, and institutional use  is free. Distribution of this code
// as  part of  a  commercial  system is  permissible  ONLY BY  DIRECT
// ARRANGEMENT WITH THE AUTHOR.
//
//  ==================================================================

#ifndef __lexer_h
#define __lexer_h

#include <iostream>
#include <sstream>
#include <fstream>
#include <cctype>
#include <string>

#include "exceptionobj.h"

namespace qmpp {

  // -----------------------------------------------------------------
  // class tLexer 
  //
  // This  class implements  a lexical  analyzer that  is used  by the
  // input mesh file reader and writer classes.
  //
  // -----------------------------------------------------------------

  class tLexer {
  public:
    // Constructor
    tLexer (istream& is, int init_linecount = -1) : 
      real_stream_(is), 
      linecount_(init_linecount) 
    {
      increment_line_(); 
    }

    // public methods
    bool get_int(int&);

    bool get_double (double&);

    int linecount() const 
      { return linecount_; }

  private:
    istream& real_stream_;
    int linecount_;

    // private methods
    void throw_error(const string& s) throw (tExceptionObject);
    void increment_line_();
    void skip_space_();
  };


};    // end namespace qmpp


#endif    // __lexer_h
