//  ===================================================================
//
//  Program:   Quadrilateral Mesh Post-Processor
//  Module:    reader.h
//  Language:  C++
//  Date:      March 2003
//  Version:   1.0.0
//
//  Copyright (c) 2002, 2003
//
//  Marcelo Siqueira
//
//  Graduate Student
//  Department of Computer and Information Science
//  Moore School Building
//  200 South 33rd Street, Suite #62
//  Philadelphia, PA 19104-6389
//  Phone No.: (215) 898-5879
//
//  marcelos@seas.upenn.edu
//
// This program  is distributed  WITHOUT ANY WARRANTY,  and it  may be
// freely redistributed under the condition that the copyright notices
// are  not  removed,  and  no  compensation  is  received.   Private,
// research, and institutional use  is free. Distribution of this code
// as  part of  a  commercial  system is  permissible  ONLY BY  DIRECT
// ARRANGEMENT WITH THE AUTHOR.
//
//  ==================================================================

#ifndef __reader_h
#define __reader_h

#include <list>
#include <vector>
#include <map>

#include <iostream>
#include <sstream>

#include "typedefs.h"
#include "exceptionobj.h"
#include "lexer.h"
#include "polygon2.h"

namespace qmpp {

  // -----------------------------------------------------------------
  // class tMeshReader
  //
  // This  class implements a reader of mesh input files.
  //
  // -----------------------------------------------------------------

  class tMeshReader {
  public:
    // Type definitions
    typedef vector<Point2>    VertexSet;
    typedef multimap<int,int> EdgeSet;
    typedef list<tPolygon2>   FaceSet;


    // Constructors
    tMeshReader(const string& fname1, const string& fname2,
		   const string& fname3) :
      filename1(fname1),
      filename2(fname2),
      filename3(fname3),
      linecount_(0)
    {}

    // Returns the value of the line counter 
    int linecount() 
      { return linecount_; }

    // Reads the vertices of a 2D mesh in a ".NODE" file
    void readVertexFile(VertexSet&) throw (tExceptionObject);

    // Reads the constrained edges of a 2D mesh in a ".EDGE" file
    void readEdgeFile(EdgeSet&) throw (tExceptionObject);
 
    // Reads the faces of a 2D mesh in a ".ELE" file
    void readElemFile(const VertexSet&, FaceSet&) throw (tExceptionObject);

  private:
    string filename1;   // name of .NODE file
    string filename2;   // name of .EDGE file
    string filename3;   // name of .ELE file
    int linecount_;     // input file line counter

  };


};    // end namespace qmpp


#endif    // __reader_h
