//  ===================================================================
//
//  Program:   Quadrilateral Mesh Post-Processor
//  Module:    writer.h
//  Language:  C++
//  Date:      March 2003
//  Version:   1.0.0
//
//  Copyright (c) 2002, 2003
//
//  Marcelo Siqueira
//
//  Graduate Student
//  Department of Computer and Information Science
//  Moore School Building
//  200 South 33rd Street, Suite #62
//  Philadelphia, PA 19104-6389
//  Phone No.: (215) 898-5879
//
//  marcelos@seas.upenn.edu
//
// This program  is distributed  WITHOUT ANY WARRANTY,  and it  may be
// freely redistributed under the condition that the copyright notices
// are  not  removed,  and  no  compensation  is  received.   Private,
// research, and institutional use  is free. Distribution of this code
// as  part of  a  commercial  system is  permissible  ONLY BY  DIRECT
// ARRANGEMENT WITH THE AUTHOR.
//
//  ==================================================================

#ifndef __writer_h
#define __writer_h

#include <list>
#include <vector>
#include <map>

#include <iostream>
#include <sstream>

#include "typedefs.h"
#include "exceptionobj.h"
#include "lexer.h"
#include "edgekey.h"
#include "polygon2.h"

namespace qmpp {

  // -----------------------------------------------------------------
  // class tMeshWriter
  //
  // This class implements a writer of mesh output files.
  //
  // -----------------------------------------------------------------
  class tMeshWriter {
  public:
    // Type definitions
    typedef map<int,Point2>    VertexSet;
    typedef map<tEdgeKey,bool> EdgeSet;
    typedef list<tPolygon2>    FaceSet;

    // Constructor
    tMeshWriter(const string& fname1, const string& fname2, 
		const string& fname3, const string& fname4) :
      filename1(fname1),
      filename2(fname2),
      filename3(fname3),
      filename4(fname4)
    {}

    // Generates the vertex file (.NODE) of a 2D mesh
    void writeVertexFile(const VertexSet&) throw (tExceptionObject);

    // Generates the edge file (.EDGE) of a 2D mesh
    void writeEdgeFile(const EdgeSet&) throw (tExceptionObject);

    // Generates the element file (.ELE) of a 2D mesh
    void writeElemFile(const FaceSet&) throw (tExceptionObject);

    // Generates the contour file (.POLY) of a 2D mesh
    void writePolyFile(const VertexSet&, const EdgeSet&) 
      throw (tExceptionObject);  

  private:
    string filename1;    // name of .NODE file
    string filename2;    // name of .EDGE file
    string filename3;    // name of .ELE file
    string filename4;    // name of .POLY file
  };


};    // end namespace qmpp


#endif    // __writer_h
