//  ===================================================================
//
//  Program:   Quadrilateral Mesh Post-Processor
//  Module:    control.h
//  Language:  C++
//  Date:      March 2003
//  Version:   1.0.0
//
//  Copyright (c) 2002, 2003
//
//  Marcelo Siqueira
//
//  Graduate Student
//  Department of Computer and Information Science
//  Moore School Building
//  200 South 33rd Street, Suite #62
//  Philadelphia, PA 19104-6389
//  Phone No.: (215) 898-5879
//
//  marcelos@seas.upenn.edu
//
// This program  is distributed  WITHOUT ANY WARRANTY,  and it  may be
// freely redistributed under the condition that the copyright notices
// are  not  removed,  and  no  compensation  is  received.   Private,
// research, and institutional use  is free. Distribution of this code
// as  part of  a  commercial  system is  permissible  ONLY BY  DIRECT
// ARRANGEMENT WITH THE AUTHOR.
//
//  ==================================================================

#ifndef __control_h
#define __control_h

#include <iostream>
#include <string>
#include <list>
#include <map>

#include <stdio.h>

#include <GL/glut.h>

#include "typedefs.h"
#include "smartptr.h"
#include "bbox2.h"
#include "hedge.h"
#include "quadmesh2.h"
#include "writer.h"
#include "edgekey.h"
#include "quadstat.h"

namespace qmpp {

  class tControl {
  public:
    // Type definitions
    typedef hedge::tComplex2::vtIterator   VIT;
    typedef hedge::tComplex2::egIterator   EIT;
    typedef hedge::tComplex2::fcIterator   FIT;
    typedef hedge::tComplex2::Pointer      cpPointer;
    typedef hedge::tVertex2::Pointer       vtPointer;
    typedef hedge::tHalfEdge2::Pointer     hePointer;
    typedef hedge::tEdge2::Pointer         egPointer;
    typedef hedge::tFace2::Pointer         fcPointer;

    // Constructors
    tControl() : 
      m_mesh(0),
      m_isImproved(false),
      m_iter(0)
    {}

    tControl(string fn, string niter) : 
      m_fname(fn),
      m_mesh(0),
      m_isImproved(false)
    {
      m_iter = (unsigned) atoi(niter.c_str());
    }

    // Destructor
    ~tControl() 
      { 
	if (m_mesh) delete m_mesh;
	m_mesh = 0;
      }

    // Drawing method
    void drawMesh();

    // Builds a quadrilateral mesh from input files
    void buildQuadMesh();

    // Improves the quality of a mesh
    void improveMesh();

    // Saves  the information  about the  quadrilateral mesh  to files
    // .NODE, .EDGE, .ELE and .POLY.
    void saveQuadrangulation();

    // Computes statistics about  the improved mesh and saves  it to a
    // file.
    void generateStats();

    // Returns a bounding box for the mesh
    tBBox2::Pointer getMeshBoundingBox()
      {
	if (m_mesh) {
	  cpPointer cp = m_mesh->getCellComplex();
	  return cp->getBoundingBox();
	}

	return tBBox2::Pointer(0);
      }

   private:
     // file name  of the  input files describing  a 2D  mesh (*.NODE,
     // *.EDGE, *.ELE files)
     std::string m_fname;

     // Pointer to the mesh builder
     tMesh* m_mesh;

     // Flag to indicate if mesh has already been improved
     bool m_isImproved;

     // Number of iterations of the mesh improvement loop
     unsigned int m_iter;
  };

};     // end namespace qmpp


#endif   // __control_h
