//  ===================================================================
//
//  Program:   Quadrilateral Mesh Post-Processor
//  Module:    glfunc.cc
//  Language:  C++
//  Date:      March 2003
//  Version:   1.0.0
//
//  Copyright (c) 2002, 2003
//
//  Marcelo Siqueira
//
//  Graduate Student
//  Department of Computer and Information Science
//  Moore School Building
//  200 South 33rd Street, Suite #62
//  Philadelphia, PA 19104-6389
//  Phone No.: (215) 898-5879
//
//  marcelos@seas.upenn.edu
//
// This program  is distributed  WITHOUT ANY WARRANTY,  and it  may be
// freely redistributed under the condition that the copyright notices
// are  not  removed,  and  no  compensation  is  received.   Private,
// research, and institutional use  is free. Distribution of this code
// as  part of  a  commercial  system is  permissible  ONLY BY  DIRECT
// ARRANGEMENT WITH THE AUTHOR.
//
//  ==================================================================

#include <GL/glut.h>

#include "glconstdefs.h"
#include "control.h"

// Extern global variable to control the execution process.
extern qmpp::tControl* control;
extern double xmin, ymin, xmax, ymax;

namespace qmpp {

  // Forward declaration of functions
  void myinit(double,double,double,double);
  void reshape(int, int);
  void display();
  void right_menu(int);

  // OpenGL init code
  void myinit(double x1, double y1, double x2, double y2)
  { 
    double dx = 0.05 * (x2 - x1);
    double dy = 0.05 * (y2 - y1);

    if (dx < 0) dx = -dx;
    if (dy < 0) dy = -dy;

    glClearColor(0.0, 0.0, 0.0, 0.0); // white background 
    glClear(GL_COLOR_BUFFER_BIT); 
    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
    gluOrtho2D(x1-dx, x2+dx, y1-dy, y2+dy);
    glMatrixMode(GL_MODELVIEW);
    glEnable(GL_POINT_SMOOTH);
    glHint(GL_POINT_SMOOTH_HINT, GL_DONT_CARE);
    glPointSize(1.5);
  }

  // OpenGL reshape callback function
  void reshape(int w, int h) 
  {
    double dx = 0.05 * (xmax - xmin);
    double dy = 0.05 * (ymax - ymin);

    if (dx < 0) dx = -dx;
    if (dy < 0) dy = -dy;

    glViewport(0,0,w,h);
    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
    gluOrtho2D(xmin-dx, xmax+dx, ymin-dy, ymax+dy);
    glMatrixMode(GL_MODELVIEW);
  }

  // OpenGL display callback function
  void display()
  {
    glClear(GL_COLOR_BUFFER_BIT);
    control->drawMesh();
    glFlush();   
  }

  // OpenGL right mouse button menu
  void right_menu(int item) {
    switch (item) {
    case M_IMPR:
      control->improveMesh();
      glutPostRedisplay();
      break;
    case M_STAT:
      control->generateStats();
      glutPostRedisplay();
      break;
    case M_SAVE:
      control->saveQuadrangulation();
      glutPostRedisplay();
      break;
    case(M_EXIT):
      if (control != 0) delete control;
      exit(0);
      break;
    }
  }

};   // end namespace qmpp
