//  ===================================================================
//
//  Program:   Quadrilateral Mesh Post-Processor
//  Module:    qmpp.cc
//  Language:  C++
//  Date:      March 2003
//  Version:   1.0.0
//
//  Copyright (c) 2002, 2003
//
//  Marcelo Siqueira
//
//  Graduate Student
//  Department of Computer and Information Science
//  Moore School Building
//  200 South 33rd Street, Suite #62
//  Philadelphia, PA 19104-6389
//  Phone No.: (215) 898-5879
//
//  marcelos@seas.upenn.edu
//
// This program  is distributed  WITHOUT ANY WARRANTY,  and it  may be
// freely redistributed under the condition that the copyright notices
// are  not  removed,  and  no  compensation  is  received.   Private,
// research, and institutional use  is free. Distribution of this code
// as  part of  a  commercial  system is  permissible  ONLY BY  DIRECT
// ARRANGEMENT WITH THE AUTHOR.
//
//  ==================================================================

#include "qmpp.h"

using namespace qmpp;

// Declaration of extern functions
namespace qmpp {
  extern void myinit(double, double, double, double);
  extern void reshape(int, int);
  extern void display();
  extern void right_menu(int);
};

// Global variable to store mesh data
tControl* control;
double xmin, ymin, xmax, ymax;

int main(int argc, char** argv)
{
  // Checks  if  user provided  a  filename  for  the files  with  the
  // vertices and elements of a mesh.
  if (argc != 3) {
    std::cout << "Usage: qmpp <mesh filename> <number of mesh quality loop iterations>" << std::endl;
    exit(0);
  }

  // Creates an  object to  control the process  of reading  the input
  // mesh and creating the quadrangulation.
  control = (tControl*) new tControl(string(argv[1]),string(argv[2]));

  std::cout << "building quadrangulation..." << std::endl;
  control->buildQuadMesh();
  // control->improveMesh();
  // control->generateStats();

  //
  // initialize a GUI to display the mesh
  //
  tBBox2::Pointer bb(control->getMeshBoundingBox());
  Point2 pmin = bb->getPMin();
  Point2 pmax = bb->getPMax();

  // get bounding box coordinates
  xmin = pmin.x();
  xmax = pmax.x();
  ymin = pmin.y();
  ymax = pmax.y();
  
  // get bounding box coordinates
  xmin = pmin.x();
  xmax = pmax.x();
  ymin = pmin.y();
  ymax = pmax.y();

  double dx = xmax - xmin + 1;
  double dy = ymax - ymin + 1;

  if (dx >= dy) {
    double dd = dy / dx;
    dx = 600;
    dy = 600 * dd;
  }
  else {
    double dd = dx / dy;
    dx = 600 * dd;
    dy = 600;
  }
  
  int xs = int(dx);
  int ys = int(dy);

  // Standard GLUT initialization 
  glutInit(&argc,argv);
  glutInitDisplayMode (GLUT_SINGLE);
  glutInitWindowSize(xs,ys);
  glutInitWindowPosition(0,0);
  glutCreateWindow("QMPP - Quadrilateral Mesh Post-Processor");

  // initialize OpenGL attributes
  myinit(xmin,ymin,xmax,ymax);

  glutDisplayFunc(display); 
  glutReshapeFunc(reshape);
  glutCreateMenu(right_menu);
  glutAddMenuEntry("Improve quadrangulation quality", M_IMPR);
  glutAddMenuEntry("Generate statistics", M_STAT);
  glutAddMenuEntry("Save quadrangulation", M_SAVE);
  glutAddMenuEntry("Exit", M_EXIT);
  glutAttachMenu(GLUT_RIGHT_BUTTON);

  // enter event loop
  glutMainLoop();
  return 0;
}
