//  ===================================================================
//
//  Program:   Quadrilateral Mesh Post-Processor
//  Module:    quad2.cc
//  Language:  C++
//  Date:      March 2003
//  Version:   1.0.0
//
//  Copyright (c) 2002, 2003
//
//  Marcelo Siqueira
//
//  Graduate Student
//  Department of Computer and Information Science
//  Moore School Building
//  200 South 33rd Street, Suite #62
//  Philadelphia, PA 19104-6389
//  Phone No.: (215) 898-5879
//
//  marcelos@seas.upenn.edu
//
// This program  is distributed  WITHOUT ANY WARRANTY,  and it  may be
// freely redistributed under the condition that the copyright notices
// are  not  removed,  and  no  compensation  is  received.   Private,
// research, and institutional use  is free. Distribution of this code
// as  part of  a  commercial  system is  permissible  ONLY BY  DIRECT
// ARRANGEMENT WITH THE AUTHOR.
//
//  ==================================================================

#include "quad2.h"

namespace qmpp {

  // -----------------------------------------------------------------
  // Method: tQuadrilateral2()
  // 
  // Default constructor
  //
  // -----------------------------------------------------------------
  tQuadrilateral2::tQuadrilateral2() : tPolygon2(4)
  {}

  // -----------------------------------------------------------------
  // Method: tQuadrilateral2()
  // 
  // Constructor
  //
  // -----------------------------------------------------------------
  tQuadrilateral2::tQuadrilateral2(const tQuadrilateral2& t) : 
    tPolygon2(4)
  {
    m_verts[0] = t.m_verts[0];
    m_verts[1] = t.m_verts[1];
    m_verts[2] = t.m_verts[2];
    m_verts[3] = t.m_verts[3];

    m_ec[0] = t.m_ec[0];
    m_ec[1] = t.m_ec[1];
    m_ec[2] = t.m_ec[2];
    m_ec[3] = t.m_ec[3];
  }

  // -----------------------------------------------------------------
  // Method: tQuadrilateral2()
  // 
  // Constructor
  //
  // -----------------------------------------------------------------
  tQuadrilateral2::tQuadrilateral2(const tPolygon2& t) 
    throw (tExceptionObject)
  {
    if (t.getNumVerts() != 4) {
      stringstream ss (stringstream::in | stringstream::out);
      ss << "Cannot convert general polygon to quadrilateral";
      throw tExceptionObject(__FILE__,__LINE__,ss.str());
    }

    m_size = 4;
    m_verts.resize(m_size);
    m_ec.resize(m_size);

    m_verts[0] = t.getIndVertex(0);
    m_verts[1] = t.getIndVertex(1);
    m_verts[2] = t.getIndVertex(2);
    m_verts[3] = t.getIndVertex(3);

    m_ec[0] = t.isConstrained(0);
    m_ec[1] = t.isConstrained(1);
    m_ec[2] = t.isConstrained(2);
    m_ec[3] = t.isConstrained(3);
  }

  // -----------------------------------------------------------------
  // Method: tQuadrilateral2()
  // 
  // Constructor
  //
  // -----------------------------------------------------------------
  tQuadrilateral2::tQuadrilateral2(tPolygon2* t) 
    throw (tExceptionObject)
  {
    if (t->getNumVerts() != 4) {
      stringstream ss (stringstream::in | stringstream::out);
      ss << "Cannot convert general polygon to quadrilateral";
      throw tExceptionObject(__FILE__,__LINE__,ss.str());
    }

    m_size = 4;
    m_verts.resize(m_size);
    m_ec.resize(m_size);

    m_verts[0] = t->getIndVertex(0);
    m_verts[1] = t->getIndVertex(1);
    m_verts[2] = t->getIndVertex(2);
    m_verts[3] = t->getIndVertex(3);

    m_ec[0] = t->isConstrained(0);
    m_ec[1] = t->isConstrained(1);
    m_ec[2] = t->isConstrained(2);
    m_ec[3] = t->isConstrained(3);
  }

  // -----------------------------------------------------------------
  // Method: =operator()
  // 
  // Overloads assignment operator
  //
  // -----------------------------------------------------------------
  tQuadrilateral2
  tQuadrilateral2::operator=(const tQuadrilateral2& t)
  {
    m_verts[0] = t.m_verts[0];
    m_verts[1] = t.m_verts[1];
    m_verts[2] = t.m_verts[2];
    m_verts[3] = t.m_verts[3];

    m_ec[0] = t.m_ec[0];
    m_ec[1] = t.m_ec[1];
    m_ec[2] = t.m_ec[2];
    m_ec[3] = t.m_ec[3];

    return *this;
  }

  // -----------------------------------------------------------------
  // Method: =operator()
  // 
  // Overloads assignment operator
  //
  // -----------------------------------------------------------------
  tQuadrilateral2
  tQuadrilateral2::operator=(const tPolygon2& t) 
    throw (tExceptionObject)
  {
    if (t.getNumVerts() != 4) {
      stringstream ss (stringstream::in | stringstream::out);
      ss << "Cannot convert general polygon to quadrilateral";
      throw tExceptionObject(__FILE__,__LINE__,ss.str());
    }

    m_size = 4;
    m_verts.resize(m_size);
    m_ec.resize(m_size);

    m_verts[0] = t.getIndVertex(0);
    m_verts[1] = t.getIndVertex(1);
    m_verts[2] = t.getIndVertex(2);
    m_verts[3] = t.getIndVertex(3);

    m_ec[0] = t.isConstrained(0);
    m_ec[1] = t.isConstrained(1);
    m_ec[2] = t.isConstrained(2);
    m_ec[3] = t.isConstrained(3);

    return *this;
  }

  // -----------------------------------------------------------------
  // Method: reverse()
  // 
  // Reverses  the  orientation  of  a  triangle.  It  overwrites  the
  // implemenation of this method in tPolygon2.
  //
  // -----------------------------------------------------------------
  void
  tQuadrilateral2::reverse()
  {
    tIndVertex2 v;

    v = m_verts[3];
    m_verts[3] = m_verts[0];
    m_verts[0] = v;

    v = m_verts[2];
    m_verts[2] = m_verts[1];
    m_verts[1] = v;

    bool aux = m_ec[0];
    m_ec[0] = m_ec[3];
    m_ec[3] = aux;

    aux = m_ec[1];
    m_ec[1] = m_ec[2];
    m_ec[2] = aux;
  }


};   // end namespace qmpp
