//  ===================================================================
//
//  Program:   Quadrilateral Mesh Post-Processor
//  Module:    stat.h
//  Language:  C++
//  Date:      March 2003
//  Version:   1.0.0
//
//  Copyright (c) 2002, 2003
//
//  Marcelo Siqueira
//
//  Graduate Student
//  Department of Computer and Information Science
//  Moore School Building
//  200 South 33rd Street, Suite #62
//  Philadelphia, PA 19104-6389
//  Phone No.: (215) 898-5879
//
//  marcelos@seas.upenn.edu
//
// This program  is distributed  WITHOUT ANY WARRANTY,  and it  may be
// freely redistributed under the condition that the copyright notices
// are  not  removed,  and  no  compensation  is  received.   Private,
// research, and institutional use  is free. Distribution of this code
// as  part of  a  commercial  system is  permissible  ONLY BY  DIRECT
// ARRANGEMENT WITH THE AUTHOR.
//
//  ==================================================================

#ifndef __stat_h
#define __stat_h

#include <string>

#include "macros.h"
#include "smartptr.h"
#include "exceptionobj.h"
#include "hedge.h"

namespace qmpp {

  // -----------------------------------------------------------------
  // class tStat
  //
  // Abstract class to produce statistical data about a general mesh.
  //
  // -----------------------------------------------------------------
  class tStat {
  public:
    // Type definition
    typedef hedge::tHalfEdge2::Pointer   hePointer;
    typedef hedge::tFace2::Pointer       fcPointer;
    typedef hedge::tComplex2::Pointer    cpPointer;
    typedef hedge::tComplex2::fcIterator FIT;

    // Constructor
    tStat() {}

    // Destructor
    virtual ~tStat() {}

    // Computes statistical information about a general mesh
    virtual void computeStats(const string&, const cpPointer&) throw (tExceptionObject) = 0;
  };


};    // end namespace qmpp


#endif   // __stat_h
