//  ===================================================================
//
//  Program:   Quadrilateral Mesh Post-Processor
//  Module:    tri2.cc
//  Language:  C++
//  Date:      March 2003
//  Version:   1.0.0
//
//  Copyright (c) 2002, 2003
//
//  Marcelo Siqueira
//
//  Graduate Student
//  Department of Computer and Information Science
//  Moore School Building
//  200 South 33rd Street, Suite #62
//  Philadelphia, PA 19104-6389
//  Phone No.: (215) 898-5879
//
//  marcelos@seas.upenn.edu
//
// This program  is distributed  WITHOUT ANY WARRANTY,  and it  may be
// freely redistributed under the condition that the copyright notices
// are  not  removed,  and  no  compensation  is  received.   Private,
// research, and institutional use  is free. Distribution of this code
// as  part of  a  commercial  system is  permissible  ONLY BY  DIRECT
// ARRANGEMENT WITH THE AUTHOR.
//
//  ==================================================================

#include "tri2.h"

namespace qmpp {

  // -----------------------------------------------------------------
  // Method: tTriangle2()
  // 
  // Default constructor
  //
  // -----------------------------------------------------------------
  tTriangle2::tTriangle2() : tPolygon2(3)
  {}

  // -----------------------------------------------------------------
  // Method: tTriangle2()
  // 
  // Constructor
  //
  // -----------------------------------------------------------------
  tTriangle2::tTriangle2(const tTriangle2& t) : tPolygon2(3)
  {
    m_verts[0] = t.m_verts[0];
    m_verts[1] = t.m_verts[1];
    m_verts[2] = t.m_verts[2];

    m_ec[0] = t.m_ec[0];
    m_ec[1] = t.m_ec[1];
    m_ec[2] = t.m_ec[2];
  }

  // -----------------------------------------------------------------
  // Method: tTriangle2()
  // 
  // Constructor
  //
  // -----------------------------------------------------------------
  tTriangle2::tTriangle2(const tPolygon2& t) throw (tExceptionObject)
  {
    if (t.getNumVerts() != 3) {
      stringstream ss (stringstream::in | stringstream::out);
      ss << "Cannot convert general polygon to triangle";
      throw tExceptionObject(__FILE__,__LINE__,ss.str());
    }

    m_size = 3;
    m_verts.resize(m_size);
    m_ec.resize(m_size);

    m_verts[0] = t.getIndVertex(0);
    m_verts[1] = t.getIndVertex(1);
    m_verts[2] = t.getIndVertex(2);

    m_ec[0] = t.isConstrained(0);
    m_ec[1] = t.isConstrained(1);
    m_ec[2] = t.isConstrained(2);
  }

  // -----------------------------------------------------------------
  // Method: tTriangle2()
  // 
  // Constructor
  //
  // -----------------------------------------------------------------
  tTriangle2::tTriangle2(tPolygon2* t) throw (tExceptionObject)
  {
    if (t->getNumVerts() != 3) {
      stringstream ss (stringstream::in | stringstream::out);
      ss << "Cannot convert general polygon to triangle";
      throw tExceptionObject(__FILE__,__LINE__,ss.str());
    }

    m_size = 3;
    m_verts.resize(m_size);
    m_ec.resize(m_size);

    m_verts[0] = t->getIndVertex(0);
    m_verts[1] = t->getIndVertex(1);
    m_verts[2] = t->getIndVertex(2);

    m_ec[0] = t->isConstrained(0);
    m_ec[1] = t->isConstrained(1);
    m_ec[2] = t->isConstrained(2);
  }

  // -----------------------------------------------------------------
  // Method: =operator()
  // 
  // Overloads assignment operator
  //
  // -----------------------------------------------------------------
  tTriangle2
  tTriangle2::operator=(const tTriangle2& t)
  {
    m_verts[0] = t.m_verts[0];
    m_verts[1] = t.m_verts[1];
    m_verts[2] = t.m_verts[2];

    m_ec[0] = t.m_ec[0];
    m_ec[1] = t.m_ec[1];
    m_ec[2] = t.m_ec[2];

    return *this;
  }

  // -----------------------------------------------------------------
  // Method: =operator()
  // 
  // Overloads assignment operator
  //
  // -----------------------------------------------------------------
  tTriangle2
  tTriangle2::operator=(const tPolygon2& t) throw (tExceptionObject)
  {
    if (t.getNumVerts() != 3) {
      stringstream ss (stringstream::in | stringstream::out);
      ss << "Cannot convert general polygon to triangle";
      throw tExceptionObject(__FILE__,__LINE__,ss.str());
    }

    m_size = 3;
    m_verts.resize(m_size);
    m_ec.resize(m_size);

    m_verts[0] = t.getIndVertex(0);
    m_verts[1] = t.getIndVertex(1);
    m_verts[2] = t.getIndVertex(2);

    m_ec[0] = t.isConstrained(0);
    m_ec[1] = t.isConstrained(1);
    m_ec[2] = t.isConstrained(2);

    return *this;
  }

  // -----------------------------------------------------------------
  // Method: isCCW()
  // 
  // Returns  true if  the triangle  is counterclockwise  oriented. It
  // overwrites the implementation of isCCW() from tPolygon2.
  //
  // -----------------------------------------------------------------
  bool
  tTriangle2::isCCW() const
  {
    Point2 p0 = m_verts[0].getPoint();
    Point2 p1 = m_verts[1].getPoint();
    Point2 p2 = m_verts[2].getPoint();

    double a1 = p1.x() - p0.x();
    double a2 = p1.y() - p0.y();
    double b1 = p2.x() - p0.x();
    double b2 = p2.y() - p0.y();

    return (a1*b2 - a2*b1) > 0;
  }

  // -----------------------------------------------------------------
  // Method: reverse()
  // 
  // Reverses  the  orientation  of  a  triangle.  It  overwrites  the
  // implemenation of this method in tPolygon2.
  //
  // -----------------------------------------------------------------
  void
  tTriangle2::reverse()
  {
    tIndVertex2 v = m_verts[0];
    m_verts[0] = m_verts[2];
    m_verts[2] = v;

    bool e = m_ec[0];
    m_ec[0] = m_ec[2];
    m_ec[2] = e;
  }


};   // end namespace qmpp
