//  ===================================================================
//
//  Program:   Quadrilateral Mesh Post-Processor
//  Module:    trimesh2.h
//  Language:  C++
//  Date:      March 2003
//  Version:   1.0.0
//
//  Copyright (c) 2002, 2003
//
//  Marcelo Siqueira
//
//  Graduate Student
//  Department of Computer and Information Science
//  Moore School Building
//  200 South 33rd Street, Suite #62
//  Philadelphia, PA 19104-6389
//  Phone No.: (215) 898-5879
//
//  marcelos@seas.upenn.edu
//
// This program  is distributed  WITHOUT ANY WARRANTY,  and it  may be
// freely redistributed under the condition that the copyright notices
// are  not  removed,  and  no  compensation  is  received.   Private,
// research, and institutional use  is free. Distribution of this code
// as  part of  a  commercial  system is  permissible  ONLY BY  DIRECT
// ARRANGEMENT WITH THE AUTHOR.
//
//  ==================================================================

#ifndef __trimesh2_h
#define __trimesh2_h

#include <string>
#include <list>
#include <map>

#include "typedefs.h"
#include "macros.h"
#include "smartptr.h"
#include "dataobj.h"
#include "exceptionobj.h"

#include "edgekey.h"
#include "polygon2.h"
#include "tri2.h"
#include "reader.h"
#include "hedge.h"
#include "mesh2.h"

namespace qmpp {

  // -----------------------------------------------------------------
  // class tTriMesh
  //
  // This  class contains  methods  to build  a  triangular mesh  from
  // .NODE, .POLY and .ELE  files containing triangle nodes, edges and
  // faces,  respectively.   The  resulting  mesh  is  stored  in  the
  // handle-edge data structure.
  //
  // -----------------------------------------------------------------
  class tTriMesh : public tMesh {
  public:
    // Default constructor
    tTriMesh() : tMesh() {};

    // Destructor
    ~tTriMesh() {}

    // Builds a triangular mesh from .NODE, .EDGE, and .ELE files, and
    // stores   the   resulting  mesh   into   the  handle-edge   data
    // structure.
    cpPointer buildMesh(const string&);

  private:
    // Creates a new  triangular face as a new  connected component of
    // the mesh.
    void makeTriFace(const tTriangle2&, int, int, int, const Point2&, 
      const Point2&, const Point2&, const tEdgeKey&, const tEdgeKey&,
      const tEdgeKey&, const ConstrainedEdgeSet&);

    // Creates a new  triangular face and attaches it  to another face
    // in the mesh.
    void makeTriFace(const tTriangle2&, int, const Point2&, 
      const tEdgeKey&, const tEdgeKey&, const tEdgeKey&, 
      const ConstrainedEdgeSet&);

    // Creates a new  triangular face as a new  connected component of
    // the mesh and  keeps track of its common  edge with another face
    // of  the mesh,  so that  we  can glue  this connected  component
    // later.
    void makeTriFaceGlue1(const tTriangle2&, int, int, int,
      const Point2&, const Point2&, const Point2&,
      const tEdgeKey&, const tEdgeKey&, const tEdgeKey&, GlueEdgeSet&,
      const ConstrainedEdgeSet&);

    // Creates a new  triangular face as a new  connected component of
    // the mesh  and keeps  track of its  two common edges  with other
    // faces of the mesh, so that we can glue this connected component
    // later.
    void makeTriFaceGlue2(const tTriangle2&, int, int, int, 
      const Point2&, const Point2&, const Point2&, const tEdgeKey&,
      const tEdgeKey&, const tEdgeKey&, int, GlueEdgeSet&,
      const ConstrainedEdgeSet&);

    // Creates a new  triangular face as a new  connected component of
    // the mesh and  keeps track of its three  common edges with other
    // faces of the mesh, so that we can glue this connected component
    // later.
    void makeTriFaceGlue3(const tTriangle2&, int, int, int,
      const Point2&, const Point2&, const Point2&, const tEdgeKey&,
      const tEdgeKey&, const tEdgeKey&, GlueEdgeSet&, 
      const ConstrainedEdgeSet&);
  };


};   // end namespace qmpp

#endif   // __trimesh2_h
